/* Linspire Clear KWin window decoration
  Copyright (C) 2004 Lindows, Inc.
            (C) 2004 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the KDE style "Plastik":

  Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the window decoration "Web":
  Copyright (C) 2001 Rik Hemsley (rikkus) <rik@kde.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
 */

#ifndef LINSPIRECLEAR_H
#define LINSPIRECLEAR_H

#include <kdecoration.h>
#include <kdecorationfactory.h>

#include <qintcache.h>
#include <qfont.h>

namespace KWinLinspireClear {

enum ButtonType {
    HelpButton=0,
    MaxButton,
    MinButton,
    CloseButton,
    MenuButton,
    OnAllDesktopsButton,
    NumButtons
};

enum ColorType {
    WindowContour=0,
    TitleGradientTo,
    TitleHighlightTop,
    SideHighlightLeft,
    SideHighlightRight,
    SideHighlightBottom,
    Border,
    TitleFont,
    BottomShade,
    TitleBlend,
    Button
};

// global pixmap cache support...
enum CacheEntryType {
    cButtonBase,
    cButtonSurface,
    cButtonIcon,
    cGradientLineHorizontal,
    cGradientLineVertical,
    cTopTile,
    cTitleBarTile,
    cBottomTile,
    cTopLeft,
    cTopRight,
    cBottomLeft,
    cBottomRight
};
struct CacheEntry
{
    CacheEntryType type;
    int width;
    int height;
    QRgb c1Rgb;
    QRgb c2Rgb;
    QRgb c3Rgb;
    bool pressed;
    int animationStep;

    QPixmap* pixmap;

    CacheEntry(CacheEntryType t, int w, int h, QRgb c1 = 0, QRgb c2 = 0, QRgb c3 = 0, bool pr = false, int as = 0, QPixmap* p = 0 ):
        type(t), width(w), height(h), c1Rgb(c1), c2Rgb(c2), c3Rgb(c3), pressed(pr), animationStep(as), pixmap(p)
    {}

    ~CacheEntry()
    {
        delete pixmap;
    }

    int key()
    {
        // create an int key from the properties which is used to refer to entries in the QIntCache.
        // the result may not be 100% correct as we don't have so much space in one integer -- use
        // == operator after find to make sure we got the right one. :)
        return pressed ^ (type<<1) ^ (width<<6) ^ (height<<17) ^ (c2Rgb<<15) ^ (c1Rgb<<20) ^ (c3Rgb<<25) ^ (animationStep<<26);
    }

    bool operator == (const CacheEntry& other)
    {
        bool match = (type == other.type) &&
                    (width   == other.width) &&
                    (height == other.height) &&
                    (c1Rgb == other.c1Rgb) &&
                    (c2Rgb == other.c2Rgb) &&
                    (c3Rgb == other.c3Rgb) &&
                    (pressed == other.pressed) &&
                    (animationStep == other.animationStep);
//         if(!match) {
//             qDebug("operator ==: conflict!");
//             qDebug("width: %d\t\tother width: %d", width, other.width);
//             qDebug("height: %d\t\tother height: %d", height, other.height);
//             qDebug("c1Rgb: %d\t\tother c1Rgb: %d", c1Rgb, other.c1Rgb);
//             qDebug("c2Rgb: %d\t\tother c1Rgb: %d", c2Rgb, other.c2Rgb);
//             qDebug("c2Rgb: %d\t\tother c1Rgb: %d", c3Rgb, other.c3Rgb);
//             qDebug("animationStep: %d\t\tother animationStep: %d", animationStep, other.animationStep);
//         }
        return match;
    }
};
typedef QIntCache<CacheEntry> PixmapCache;

class LinspireClearHandler: public QObject, public KDecorationFactory
{
    Q_OBJECT
public:
    LinspireClearHandler();
    virtual ~LinspireClearHandler();

    virtual bool reset( unsigned long changed );

    virtual KDecoration* createDecoration( KDecorationBridge* );
#if (KDE_VERSION >= KDE_MAKE_VERSION(3,3,89) ) // requires at least KDE CVS/3.3.89
    bool supports( Ability ability );
#endif

    static int  titleHeight() { return m_titleHeight; }
    static int  titleHeightTool() { return m_titleHeightTool; }
    static QFont titleFont() { return m_titleFont; }
    static QFont titleFontTool() { return m_titleFontTool; }
    static bool titleShadow() { return m_titleShadow; }
    static int  borderSizeSides() { return m_borderSizeSides; }
    static int  borderSizeBottom() { return m_borderSizeBottom; }
    static bool animateButtons() { return m_animateButtons; }
    static bool menuClose() { return m_menuClose; }
    static Qt::AlignmentFlags titleAlign() { return m_titleAlign; }
    static bool reverseLayout() { return m_reverse; }
    static QColor getColor(KWinLinspireClear::ColorType type, const bool active = true);

    PixmapCache& pixmapCache() { return (*m_pixmapCache); }

    QValueList< LinspireClearHandler::BorderSize >  borderSizes() const;
private:
    void readConfig();

    static bool m_titleShadow;
    static bool m_shrinkBorders;
    static bool m_animateButtons;
    static bool m_menuClose;
    static bool m_reverse;
    static int  m_borderSizeSides;
    static int  m_borderSizeBottom;
    static int  m_titleHeight;
    static int  m_titleHeightTool;
    static QFont m_titleFont;
    static QFont m_titleFontTool;
    static Qt::AlignmentFlags m_titleAlign;

    // pixmap cache support
    PixmapCache *m_pixmapCache;
};

LinspireClearHandler* Handler();

} // KWinLinspireClear

#endif // LINSPIRECLEAR_H
