/* Linspire Clear KWin window decoration
  Copyright (C) 2004 Lindows, Inc.
            (C) 2004 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the KDE style "Plastik":

  Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the window decoration "Web":
  Copyright (C) 2001 Rik Hemsley (rikkus) <rik@kde.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
 */

#include <qbitmap.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <kimageeffect.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <qtooltip.h>
#include <qtimer.h>

#include "embed.h"

#include "linspireclearbutton.h"
#include "linspireclearbutton.moc"
#include "linspireclearclient.h"
#include "misc.h"

namespace KWinLinspireClear
{

static const uint TIMERINTERVAL = 50; // msec
static const uint ANIMATIONSTEPS = 4;

LinspireClearButton::LinspireClearButton(LinspireClearClient *parent, const char *name,
                             const QString& tip, ButtonType type, int size, int btns)
    : QButton(parent->widget(), name),
    m_client(parent),
    m_lastMouse(NoButton),
    m_realizeButtons(btns),
    m_type(type),
    hover(false),
    isOnAllDesktops(false),
    isMaximized(false)
{
    QToolTip::add( this, tip );
    setCursor(ArrowCursor);

    setBackgroundMode(NoBackground);

    setSize(size);

    m_animTmr = new QTimer(this);
    connect(m_animTmr, SIGNAL(timeout() ), this, SLOT(animate() ) );
    m_animProgress = 0;
}

LinspireClearButton::~LinspireClearButton()
{
}

void LinspireClearButton::setSize(int s)
{
    if(s < 12) { s = 12; }
    int w = s-1;
//     if( w%2 != 0 )
//         --w;
    int h = s;

    setFixedSize(w, h);
}

inline QPixmap* LinspireClearButton::getPixmap(CacheEntryType type)
{
    bool active = m_client->isActive();
    QColor buttonColor = LinspireClearHandler::getColor(Button, active);
    QColor c1 = active? hsvRelative(buttonColor, +10, +70 /*-50*/, -80 /*-100*/) : hsvRelative(buttonColor, +5, +25, -45);

    bool pressed = isDown();
    int animStep = m_animProgress;
    QColor c2 = pressed ? buttonColor.dark(100+animStep*5) : buttonColor.light(100+animStep*5);
//     if( m_type == CloseButton ) {
//         int highlightAlpha = static_cast<int>(255-((160/static_cast<double>(ANIMATIONSTEPS))*static_cast<double>(animStep) ) );
//         c2 = alphaBlendColors( c2, Qt::red, highlightAlpha );
//     }

//     int w = height();
//     if( w%2 != 0 )
//         --w;
    int w = width();
    int h = w-1;

    PixmapCache& cache = Handler()->pixmapCache();

    if( type == cButtonBase ) {

        // set up a gradient in tempTexture
        QColor shadeColor1 = hsvRelative(c1, 0, -140, +10)/*.dark(120)*/;
        QColor shadeColor2 = hsvRelative(c1, 0, -125, +70);
        if( active )
            shadeColor2 = shadeColor2.dark(115);
        else
            shadeColor2 = shadeColor2.light(115);
//         QColor shadeColor1 = LinspireClearHandler::getColor(TitleGradientTo).dark(110);
//         QColor shadeColor2 = LinspireClearHandler::getColor(TitleHighlightTop).light(110);

        // generate a quite unique cache key for this pixmap.
        CacheEntry search( cButtonBase, width(), height(), c1.rgb(), shadeColor1.rgb(), shadeColor2.rgb() /*aLightColor.rgb()*/ /*, aShadeColor.rgb()*/ );
        int key = search.key();

        CacheEntry *cacheEntry;
        if( (cacheEntry = cache.find(key)) ) {
            if( search == *cacheEntry ) { // match! we can return it now...
                if(cacheEntry->pixmap)
                    return cacheEntry->pixmap;
            } else {
                // Remove old entry in case of a conflict!
                // This shouldn't happen very often, see comment in CacheEntry.
                cache.remove(key);
            }
        }

        // ...nothing, construct it now.
        QImage base, tempTexture;
        base = uic_findImage("button_base.png").smoothScale(w,h);

//         tempTexture = KImageEffect::gradient( QSize(base.width(), base.height()),
//                                             shadeColor1, shadeColor2,
//                                             KImageEffect::VerticalGradient );
//         for(int x = 0; x < base.width(); ++x) {
//             for(int y = 0; y < base.height(); ++y) {
//                 QRgb oldRgb = base.pixel(x,y);
//                 QRgb newRgb = tempTexture.pixel(x,y);
//                 // trick 17: paint with old alpha and new color data...
//                 int alpha = qAlpha(oldRgb);
//                 int red   = qRed(newRgb);
//                 int green = qGreen(newRgb);
//                 int blue  = qBlue(newRgb);
//                 base.setPixel(x,y, qRgba(red, green, blue, alpha) );
//             }
//         }

//         QColor shadeColor2 = LinspireClearHandler::getColor(TitleHighlightTop,active);
        int h1 = base.height()/2;
        // first half of the gradient
        tempTexture = KImageEffect::gradient( QSize(base.width(), h1),
                                            c1, shadeColor1,
                                            KImageEffect::VerticalGradient );
        // and fill the base with the texture, preserving the alpha.
        for(int x = 0; x < base.width(); ++x) {
            for(int y = 0; y < h1; ++y) {
                QRgb oldRgb = base.pixel(x,y);
                QRgb newRgb = tempTexture.pixel(x,y);
                // trick 17: paint with old alpha and new color data...
                int alpha = qAlpha(oldRgb);
                int red   = qRed(newRgb);
                int green = qGreen(newRgb);
                int blue  = qBlue(newRgb);
                base.setPixel(x,y, qRgba(red, green, blue, alpha) );
            }
        }
        // second one
        tempTexture = KImageEffect::gradient( QSize(base.width(), base.height()-h1),
                                            shadeColor1, shadeColor2,
                                            KImageEffect::VerticalGradient );
        // and fill the base with the texture, preserving the alpha.
        for(int x = 0; x < base.width(); ++x) {
            for(int y = 0; y < (base.height()-h1); ++y) {
                QRgb oldRgb = base.pixel(x,h1+y);
                QRgb newRgb = tempTexture.pixel(x,y);
                // trick 17: paint with old alpha and new color data...
                int alpha = qAlpha(oldRgb);
                int red   = qRed(newRgb);
                int green = qGreen(newRgb);
                int blue  = qBlue(newRgb);
                base.setPixel(x,h1+y, qRgba(red, green, blue, alpha) );
            }
        }


        QPixmap *result = new QPixmap(base);

        // insert into cache using the previously created key.
        CacheEntry *toAdd = new CacheEntry(search);
        toAdd->pixmap = result;
        cache.insert( key, toAdd, result->width()*result->height()*result->depth()/8 );

        return result;
    } else if( type == cButtonSurface ) {

        QColor c3 = alphaBlendColors(c2,Qt::white, 20)/*.light(115)*/;
        QColor c4 = hsvRelative(c2, -35, -40, +15)/*.dark(115)*/;
        if(!active)
            c4 = c4.dark(115);

        CacheEntry search( cButtonSurface, width(), height(), c2.rgb(), c3.rgb(), c4.rgb(), pressed, animStep );
        int key = search.key();

        CacheEntry *cacheEntry;
        if( (cacheEntry = cache.find(key)) ) {
            if( search == *cacheEntry ) { // match! we can return it now...
                if(cacheEntry->pixmap)
                    return cacheEntry->pixmap;
            } else {
                // Remove old entry in case of a conflict!
                // This shouldn't happen very often, see comment in CacheEntry.
                cache.remove(key);
            }
        }

        // ...nothing, construct it now.
        QImage surface, tempTexture;
        surface = uic_findImage("button_surface.png").smoothScale(w-2, h-2);

        // set up the gradient...
//         QColor c3 = c2.light(140);
//         QColor c4 = c2.dark(110);

//         tempTexture = KImageEffect::gradient( QSize(surface.width(), surface.height()),
//                                             pressed?c4:c3, pressed?c3:c4,
//                                             KImageEffect::VerticalGradient );
//         for(int x = 0; x < surface.width(); ++x) {
//             for(int y = 0; y < surface.height(); ++y) {
//                 QRgb oldRgb = surface.pixel(x,y);
//                 QRgb newRgb = tempTexture.pixel(x,y);
//                 // trick 17: paint with old alpha and new color data...
//                 int alpha = qAlpha(oldRgb);
//                 int red   = qRed(newRgb);
//                 int green = qGreen(newRgb);
//                 int blue  = qBlue(newRgb);
//                 surface.setPixel(x,y, qRgba(red, green, blue, alpha) );
//             }
//         }

        int h1 = surface.height()/2;
        int h2 = surface.height()-h1;
        // first part of the gradient:
        tempTexture = KImageEffect::gradient( QSize(surface.width(), h1),
                                            pressed?c4:c3, c2,
                                            KImageEffect::VerticalGradient );
        for(int x = 0; x < surface.width(); ++x) {
            for(int y = 0; y < h1; ++y) {
                QRgb a = surface.pixel(x,y);
                QRgb b = tempTexture.pixel(x,y);
                int alpha = qAlpha(a);
                int red   = qRed(b);
                int green = qGreen(b);
                int blue  = qBlue(b);
                QRgb c = alphaBlendColors( QColor(qRgb(red, green, blue)), Qt::white, 255-qGreen(a) ).rgb();
                surface.setPixel(x,y, qRgba(qRed(c), qGreen(c), qBlue(c), alpha) );
            }
        }
        // second part of the gradient:
        tempTexture = KImageEffect::gradient( QSize(surface.width(), h2),
                                            c2, pressed?c3:c4,
                                            KImageEffect::VerticalGradient );
        for(int x = 0; x < surface.width(); ++x) {
            for(int y = 0; y < h2; ++y) {
                QRgb a = surface.pixel(x,h1+y);
                QRgb b = tempTexture.pixel(x,y);
                int alpha = qAlpha(a);
                int red   = qRed(b);
                int green = qGreen(b);
                int blue  = qBlue(b);
                QRgb c = alphaBlendColors( QColor(qRgb(red, green, blue)), Qt::white, 255-qGreen(a) ).rgb();
                surface.setPixel(x,h1+y, qRgba(qRed(c), qGreen(c), qBlue(c), alpha) );
            }
        }

        QPixmap *result = new QPixmap(surface);

        // insert into cache using the previously created key.
        CacheEntry *toAdd = new CacheEntry(search);
        toAdd->pixmap = result;
        cache.insert( key, toAdd, result->width()*result->height()*result->depth()/8 );

        return result;
    } else /* if( type == cButtonIcon )*/ { // case default should never be happening, but to make the compiler happy...
        // store m_type, isMaximized, isOnAllDesktop in two QRgbs which are also represented as int.
        CacheEntry search( cButtonIcon, width(), height(), c1.rgb(), m_type, isMaximized ^ (isOnAllDesktops<<1) );
        int key = search.key();

        CacheEntry *cacheEntry;
        if( (cacheEntry = cache.find(key)) ) {
            if( search == *cacheEntry ) { // match! we can return it now...
                if(cacheEntry->pixmap)
                    return cacheEntry->pixmap;
            } else {
                // Remove old entry in case of a conflict!
                // This shouldn't happen very often, see comment in CacheEntry.
                cache.remove(key);
            }
        }

        // ...nothing, construct it now.
        QImage icon;
        switch( m_type ) {
            case MaxButton:
                icon = uic_findImage(isMaximized?"icon_restore.png":"icon_maximize.png").smoothScale(w-2, h-2);
                break;
            case MinButton:
                icon = uic_findImage("icon_minimize.png").smoothScale(w-2, h-2);
                break;
            case CloseButton:
                icon = uic_findImage("icon_close.png").smoothScale(w-2, h-2);
                break;
            case HelpButton:
                icon = uic_findImage("icon_help.png").smoothScale(w-2, h-2);
                break;
            case MenuButton:
                icon = uic_findImage("icon_menu.png").smoothScale(w-2, h-2);
                break;
            case OnAllDesktopsButton:
                icon = uic_findImage(isOnAllDesktops?"icon_noad.png":"icon_oad.png").smoothScale(w-2, h-2);
                break;
            default:
                ; // nothing...
        }

        // set up the gradient...

//         QColor &c5 = c1;
        QColor c5 = LinspireClearHandler::getColor(TitleFont, active);

        QPixmap *result = new QPixmap( recolorImage( icon, c5 ) );

        // insert into cache using the previously created key.
        CacheEntry *toAdd = new CacheEntry(search);
        toAdd->pixmap = result;
        cache.insert( key, toAdd, result->width()*result->height()*result->depth()/8 );

        return result;
    }
}

void LinspireClearButton::setTipText(const QString &tip) {
    QToolTip::remove(this );
    QToolTip::add(this, tip );
}

void LinspireClearButton::animate()
{
    m_animTmr->stop();

    if(hover) {
        if(m_animProgress < ANIMATIONSTEPS) {
            if (LinspireClearHandler::animateButtons() ) {
                m_animProgress++;
            } else {
                m_animProgress = ANIMATIONSTEPS;
            }
            m_animTmr->start(TIMERINTERVAL, true); // single-shot
        }
    } else {
        if(m_animProgress > 0) {
            if (LinspireClearHandler::animateButtons() ) {
                m_animProgress--;
            } else {
                m_animProgress = 0;
            }
            m_animTmr->start(TIMERINTERVAL, true); // single-shot
        }
    }

    repaint(false);
}

void LinspireClearButton::enterEvent(QEvent *e)
{
    QButton::enterEvent(e);

    hover = true;
    animate();
}

void LinspireClearButton::leaveEvent(QEvent *e)
{
    QButton::leaveEvent(e);

    hover = false;
    animate();
}

void LinspireClearButton::mousePressEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   (e->button()&m_realizeButtons)?LeftButton:NoButton, e->state());

    QButton::mousePressEvent(&me);
}

void LinspireClearButton::mouseReleaseEvent(QMouseEvent* e)
{
    m_lastMouse = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                    (e->button()&m_realizeButtons)?LeftButton:NoButton, e->state());

    QButton::mouseReleaseEvent(&me);
}

void LinspireClearButton::drawButton(QPainter *painter)
{
    QRect r(0,0,width(),height());

    bool active = m_client->isActive();
    QPixmap* backgroundTile = m_client->getTitleBarTile(active);

    QPixmap buffer(width(), height());
    QPainter bP(&buffer);

    // fill with the titlebar background
    bP.drawTiledPixmap(0, 0, width(), height(), *backgroundTile);

    QPixmap *base = getPixmap(cButtonBase),
            *surface = getPixmap(cButtonSurface);
    // the button should be drawn at the bottom...
    QRect br = QRect(r.x(), r.bottom() - base->height(), r.width(), base->height() );
    int dX = br.x()+(br.width()-base->width())/2;
    int dY = br.y()+(br.height()-base->height())/2;
    bP.drawPixmap(dX, dY, *base );
    dX = br.x()+(br.width()-surface->width())/2;
    dY = br.y()+(br.height()-surface->height())/2;
    bP.drawPixmap(dX, dY,  *surface );
    if( active || hover ) {
        QPixmap *icon = getPixmap(cButtonIcon);
        bP.drawPixmap(dX, isDown()?dY+1:dY, *icon);
    }

    bP.end();
    painter->drawPixmap(0, 0, buffer);
}

} // KWinLinspireClear
