/* Linspire Clear KWin window decoration
  Copyright (C) 2004 Lindows, Inc.
            (C) 2004 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the KDE style "Plastik":

  Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

  based on the window decoration "Web":
  Copyright (C) 2001 Rik Hemsley (rikkus) <rik@kde.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
 */

#ifndef LINSPIRECLEARCLIENT_H
#define LINSPIRECLEARCLIENT_H

#include <kdecoration.h>
#include "linspireclear.h"

class QSpacerItem;
class QVBoxLayout;
class QBoxLayout;

namespace KWinLinspireClear {

class LinspireClearButton;

class LinspireClearClient : public KDecoration
{
    Q_OBJECT
public:
    LinspireClearClient(KDecorationBridge* bridge, KDecorationFactory* factory);
    ~LinspireClearClient();

    virtual void init();

    virtual void borders( int& left, int& right, int& top, int& bottom ) const;
    virtual void resize(const QSize&);
    virtual QSize minimumSize() const;
    virtual void show();
    virtual bool eventFilter( QObject* o, QEvent* e );

    QPixmap* getTitleBarTile(bool active) const
    {
        QPixmap* tile = getPixmap(TitleBar_Tile_Pixmap, active);
        return tile;
    }
protected:
    virtual void resizeEvent();
    virtual void paintEvent(QPaintEvent *e);
    virtual void mouseDoubleClickEvent(QMouseEvent *e);

    virtual void maximizeChange();
    virtual void desktopChange();
    virtual void shadeChange() {};
    virtual void doShape();

    virtual void reset( unsigned long changed );

    virtual void captionChange();
    virtual void iconChange();
    virtual void activeChange();
    virtual Position mousePosition(const QPoint &point) const;

private slots:
    void slotMaximize();
    void menuButtonPressed();
    void menuButtonReleased();
    bool isTool();
private:
    enum ClientPixmapType {
        Top_Tile_Pixmap,
        TitleBar_Tile_Pixmap,
        Bottom_Tile_Pixmap,
        TopLeft_Pixmap,
        TopRight_Pixmap,
        BottomLeft_Pixmap,
        BottomRight_Pixmap
    };
    QPixmap* getPixmap(ClientPixmapType type, bool active) const;
    inline void renderSoftBlend(QPainter *p, const QColor &bg, const QColor &fg, int x, int y) const;

    void _resetLayout();
    void addButtons(QBoxLayout* layout, const QString& buttons, int buttonSize = 18);

    QVBoxLayout *mainLayout_;
    QSpacerItem *topSpacer_,
                *titleSpacer_,
                *leftTitleSpacer_, *rightTitleSpacer_,
                *decoSpacer_,
                *leftSpacer_, *rightSpacer_,
                *bottomSpacer_;

    QPixmap *aCaptionBuffer, *iCaptionBuffer;
    void update_captionBuffer();

    LinspireClearButton *m_button[NumButtons];

    bool captionBufferDirty;

    bool closing;

    // settings...
    int   s_titleHeight;
    QFont s_titleFont;
};

} // KWinLinspireClear

#endif // LINSPIRECLEARCLIENT_H
