/*
Copyright (C) 2004 Lindows, Inc.
          (C) 2004 Sandro Giessl <ceebx@users.sourceforge.net>

based on the KDE style "Plastik":

Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

based on the Keramik configuration dialog:
Copyright (c) 2003 Maksim Orlovich <maksim.orlovich@kdemail.net>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/

#include <qcheckbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qsettings.h>
#include <kglobal.h>
#include <klocale.h>
#include <qwhatsthis.h>

#include "linspireclearconf.h"

extern "C"
{
    QWidget* allocate_kstyle_config(QWidget* parent)
    {
        KGlobal::locale()->insertCatalogue("kstyle_linspireclear_config");
        return new LinspireClearStyleConfig(parent);
    }
}

LinspireClearStyleConfig::LinspireClearStyleConfig(QWidget* parent): QWidget(parent)
{
    //Should have no margins here, the dialog provides them
    QVBoxLayout* layout = new QVBoxLayout(this, 0, 0);
    KGlobal::locale()->insertCatalogue("kstyle_linspireclear_config");

//     animateProgressBar = new QCheckBox(i18n("Animate progress bars"), this);
    drawToolBarSeparator = new QCheckBox(i18n("Draw toolbar separator"), this);
    mouseOver = new QCheckBox(i18n("High&light widgets under mouse"), this);
    QHBox *hbox =  new QHBox(this);
    QSpacerItem *spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
    hbox->layout()->addItem(spacer);
    mouseOverAnimation = new QCheckBox(i18n("Enable animation"), hbox);

    QWhatsThis::add( mouseOver, i18n("If this option is selected, buttons will change "
            "their color when the mouse cursor is moved over them." ) );

    layout->add(drawToolBarSeparator);
    layout->add(mouseOver);
    layout->add(hbox);
    layout->addStretch(1);

    QSettings s;
    origDrawToolBarSeparator = s.readBoolEntry("/linspireclearstyle/Settings/drawToolBarSeparator", false);
    drawToolBarSeparator->setChecked(origDrawToolBarSeparator);
    origMouseOver = s.readBoolEntry("/linspireclearstyle/Settings/mouseOver", true);
    mouseOver->setChecked(origMouseOver);
    origMouseOverAnimation = s.readBoolEntry("/linspireclearstyle/Settings/mouseOverAnimation", true);
    mouseOverAnimation->setChecked(origMouseOverAnimation);
    mouseOverAnimation->setEnabled(mouseOver->isChecked());


    connect(drawToolBarSeparator, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
    connect(mouseOver, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );
    connect(mouseOver, SIGNAL( toggled(bool) ), mouseOverAnimation, SLOT( setEnabled(bool) ) );
    connect(mouseOverAnimation, SIGNAL( toggled(bool) ), SLOT( updateChanged() ) );

}

LinspireClearStyleConfig::~LinspireClearStyleConfig()
{
}


void LinspireClearStyleConfig::save()
{
    QSettings s;
    s.writeEntry("/linspireclearstyle/Settings/drawToolBarSeparator", drawToolBarSeparator->isChecked());
    s.writeEntry("/linspireclearstyle/Settings/mouseOver", mouseOver->isChecked());
    s.writeEntry("/linspireclearstyle/Settings/mouseOverAnimation", mouseOverAnimation->isChecked());
}

void LinspireClearStyleConfig::defaults()
{
    drawToolBarSeparator->setChecked(false);
    mouseOver->setChecked(true);
    mouseOverAnimation->setChecked(true);
    //updateChanged would be done by setChecked already
}

void LinspireClearStyleConfig::updateChanged()
{
    if ( (drawToolBarSeparator->isChecked() == origDrawToolBarSeparator) &&
            (mouseOver->isChecked() == origMouseOver) &&
            (mouseOverAnimation->isChecked() == origMouseOverAnimation) )
        emit changed(false);
    else
        emit changed(true);
}

#include "linspireclearconf.moc"
